#include "envy.h"
#include "envy_config.h"
#include "envy_osdb.h"
#include "envy_options.h"
#include "envy_subtitles.h"
#include "envy_engine.h"
#include <libgen.h>

static void _print_usage(const char *cmd);
static void _print_avaible_engines(const char *prefix);

const char * 
envy_parse_opt(int argc, char **argv, Envy_Config *ec)
{
  char c;
  struct option long_options[] =
    {
      {"sub"        ,        required_argument, 0, 's'},
      {"osdb"       ,              no_argument, 0, 'o'},
      {"evas-engine",        required_argument, 0, 'e'},
      {"help"       ,              no_argument, 0, 'h'},
      { 0           ,              no_argument, 0,  0 }
    };
  int option_index = 0;
  int osdb = 0;
  char *evas_engine = NULL;
  char *subfile = NULL;

  while (1)
    {
      Envy_Osdb *eo;
      c = getopt_long(argc, argv, "s:oe:h",
                      long_options, &option_index);
  
      if (c == -1)
        break;
  
      switch (c)
        {
        case 's':
          subfile = optarg;
          break;
        case 'o':
          osdb = 1;
          break;
        case 'e':
          evas_engine = optarg;
          break;
        case 'h':
          _print_usage(argv[0]);
          exit(EXIT_SUCCESS);
          break;
        default:
          _print_usage(argv[0]);
          exit(EXIT_FAILURE);
        }
    }

  envy->filename = argv[argc-1];
  envy->dirname = strdup(envy->filename);
  envy->dirname = dirname(envy->dirname);

  if (evas_engine)
    {
      int engine_type = envy_engine_type(evas_engine);
      if (engine_type != -1)
        ec->ecore_evas_engine_type = (Ecore_Evas_Engine_Type) engine_type;
      else
        {
          printf("Unknow Evas %s engine.\n", evas_engine);
          exit(EXIT_FAILURE);
        }

    }

  if (!subfile)
    {
      char *p;
      subfile = calloc(strlen(envy->filename)+1, sizeof(char));
      strcpy(subfile, envy->filename);
      p = &subfile[strlen(subfile) - 1];
      while (p)
        {
          p--;
          if (*p == '/') break;
          if (*p == '.')
            {
              p++;
              *p = '\0';
              strcat(subfile, "srt");
              if (!access(subfile, R_OK))
                {
                  envy_subtitles_load(subfile);
                  return envy->filename;
                }
              else
                {
                  free(subfile);
                  subfile = NULL;
                  break;
                }
            }
        }
    }
  else
    {
      envy_subtitles_load(subfile);
    }

  if (osdb || (ec && ec->auto_osdb))
    {
      envy_osdb_auto_search(ec->langage, ec->keep_osdb_subfilename);
    }

  return argv[argc-1];
}

static void 
_print_usage(const char *cmd)
{
  printf("Usage: %s [-oh] [-s subfile] [-e engine] filename\n", cmd);
  printf("  -s, --sub         "
         "Load subfile for the subtitles \n");
  printf("  -o, --osdb        "
         "Try to download the subfile on OpenSubtitles.org\n");
  printf("  -e, --evas-engine "
         "Use a specific evas engine:\n");
  _print_avaible_engines("                     ");
  printf("  -h, --help        "
         "Display this message\n");
}

static void
_print_avaible_engines(const char *prefix)
{
  const char ** engines = NULL;
  size_t i;

  engines = envy_engine_names();
  for (i = 0; engines[i] != NULL; i++)
    printf("%s %s\n", prefix, engines[i]);
  free(engines);
}
