#include "envy.h"
#include "envy_subtitles.h"

static void parse_srt(FILE *fd);
static int  sub_comp(const void *d1, const void *d2);

void
envy_subtitles_load(const char *subfile)
{
  FILE *fd;
  fd = fopen(subfile, "r");
  if (fd == NULL)
    {
      perror("Reading subfile");
      return;
    }
  parse_srt(fd);
  fclose(fd);
}

Sub_Srt *
envy_subtitle_get(Sub_Srt *subs, int subs_size, double time)
{
  Sub_Srt key, *current_sub = NULL;
  key.start = time;
  current_sub = bsearch(&key, subs, subs_size, sizeof(Sub_Srt), sub_comp);

  return current_sub;
}

static int
sub_comp(const void *d1, const void *d2)
{
  const Sub_Srt *sub1, *sub2;
  sub1 = d1;
  sub2 = d2;
  if (sub2->start <= sub1->start && sub2->end > sub1->start)
    return 0;
  else if (sub2->start <= sub1->start && sub2->end < sub1->start)
    return 1;
  else 
    return -1;
}

static void
parse_srt(FILE *fd)
{
  char c = '\0';
  char *id_str;
  char *start_str;
  char *end_str;
  int i = 0;
  int id_size = 5;

  id_str    = calloc(id_size, sizeof(char));
  start_str = calloc(13, sizeof(char));
  end_str   = calloc(13, sizeof(char));

  if (fd == NULL) return;

  /* Get to the end */
  fseek(fd, 0, SEEK_END);
  /* Skip the last empty line */
  for (c = getc(fd); c == '\r' || c == '\n' || c == EOF; ungetc(c, fd), fseek(fd, -1L, SEEK_CUR)) c = getc(fd);
  /* Get back until we find a blank line */
  while (i != 3)
    {
      fseek(fd, -1L, SEEK_CUR);
      c = getc(fd);
      if ('\r' == c || '\n' == c)
        i++;
      else
        i = 0;
      ungetc(c, fd);
    }
  fgetc(fd);
  fgetc(fd);
  /* The last id will serv as base size for the subs sarray */
  fscanf(fd, "%d\r\n", &i);
  rewind(fd);
  if (i > 0) envy->subs_size = i;
  envy->subs = malloc(envy->subs_size * sizeof(Sub_Srt));
//  printf("%d\n", envy->subs_size);
  i = 0;

  while (!feof(fd))
    {
      int ret;
      
      ret = fscanf(fd, "%d\r\n", &(envy->subs[i].id));
      if (ret == EOF) break;
//      printf("id    : %d\n", envy->subs[i].id );

      int start_hour, start_min, start_sec, start_ms;
      int end_hour, end_min, end_sec, end_ms;
      envy->subs[i].start = 0;
      ret = fscanf(fd, "%d:%d:%d,%d --> %d:%d:%d,%d\r\n", 
                   &start_hour, &start_min, &start_sec, &start_ms,
                   &end_hour, &end_min, &end_sec, &end_ms);
      if (ret == EOF) break;
      envy->subs[i].start = 0;
      envy->subs[i].start += start_hour*3600;
      envy->subs[i].start += start_min*60;
      envy->subs[i].start += start_sec;
      envy->subs[i].start += start_ms/1000.;
      envy->subs[i].end = 0;
      envy->subs[i].end += end_hour*3600;
      envy->subs[i].end += end_min*60;
      envy->subs[i].end += end_sec;
      envy->subs[i].end += end_ms/1000.;
//      printf("start : %f\n",  envy->subs[i].start);
//      printf("end   : %f\n",  envy->subs[i].end);

      char str[256];
      unsigned int txt_size = 0;
      int len = 0;
      char *txt = NULL;
      int j = 0;
      txt = calloc(256, sizeof(char));
      while ((len = fscanf(fd, "%255[^\r\n]%*c%*c", str)) != 0 && len != EOF)
        {
          if (txt_size == 0)
            {
              txt = strcpy(txt, str);
              txt_size = strlen(str);
            }
          else
            {
              if (txt_size + strlen(str) + 5 > 256)
                {
                  fprintf(stderr, "What the fuck ? More than 256char on a line !\n");
                  break;
                }
              txt = strcat(txt, "<br>");
              txt = strcat(txt, str);
              txt_size += strlen(str) + 4;
            }
        } 
      txt = realloc(txt, txt_size + 1);
      envy->subs[i].txt = txt;
//      printf("txt   : %s\n",  envy->subs[i].txt);

      i++;
    }
}
